*! v1.0.1  arb/pr 21dec98
*! graph proportions
program define pgraph
	version 5.0
	local varlist "req exist min(2) max(2)"
	local if "opt"
	local in "opt"
	local options "BIN(int 0) Connect(str) Symbol(str) CI LOGit XLALl Exact *"
	parse "`*'"
	if `bin'!=0 & (`bin'<2 | `bin'>100) {
		di in red "invalid bin(), out of range 2-100"
		exit 198
	}
	if "`ci'"!="" | "`exact'"!="" {
		tempvar se low high
	}
	if "`symbol'"=="" {
		local symbol o
		if "`ci'"!="" { local symbol `symbol'ii }
	}
	if "`connect'"=="" {
		local connect l
		if "`ci'"!="" { local connect `connect'II }
	}
	tempvar touse
	mark `touse' `if' `in'
	markout `touse' `varlist'
	quietly {
		preserve
		drop if `touse'==0
		drop `touse'
		parse "`varlist'", parse(" ")
		local yvar `1'
		local xvar `2'
		inspect `yvar'
		if _result(7)<1 | _result(7)>2 {
			noi di in red "`yvar' is not binary"
			error 499
		}
		sum `yvar', meanonly
		replace `yvar'=(`yvar'==_result(6))       /* make var 0/1 variable */
		local succode = _result(6)		  /* remember `success' code */
		if `bin'>0 {
			inspect `xvar'
			if _result(7)<`bin' {
				noi di in red "too many bins"
				exit 198
			}
			tempvar collvar
			xtile3 `collvar'=`xvar', nq(`bin')
		}
		else local collvar `xvar'
		tempvar r n y top
		collapse (count) `n'=`yvar' (sum) `r'=`yvar', by(`collvar') fast
                * Label all x values:
                if "`xlall'" != "" {
                        inspect `collvar'
                        local nvals = _result(7)
                        if `nvals' > 25 {
                                di in red "xlall option cannot label more than 25 values"
                                exit 499
                        }
                        sort `collvar'
                        by `collvar': gen `top'=1 if _n==1 & `collvar'!=.
                        sort `top'
                        local i 1
                        local xlall xlabel(
                        while `i' < `nvals' {
                                local val = `collvar'[`i']
                                local xlall "`xlall'`val',"
                                local i = `i' + 1
                        }
                        local val = `collvar'[`i']
                        local xlall "`xlall'`val')"
                }

/*
	Empirical logits, see Cox and Snell Analysis of Binary Data (2nd ed)
	(1989) page 32.
*/
		gen `y'=cond("`logit'"!="", /*
		 */ cond(`n'>0, ln((`r'+.5)/(`n'-`r'+.5)), .), /*
		 */ cond(`n'>0, `r'/`n', .) )
		if "`ci'"!="" {
                        if "`exact'"=="" {
				gen `se'=sqrt(cond(`n'>0, /*
				 */ (`n'+1)*(`n'+2)/(`n'*(`r'+1)*(`n'-`r'+1)), .))
				tempname z
				scalar `z'=-invnorm((100-$S_level)/200)
                        }
                        else {
                                local i 1
                                gen `low'=.
                                gen `high'=.
                                while `i'<=_N {
                                        local ni=`n'[`i']
                                        local ri=`r'[`i']
                                        cii `ni' `ri'
                                        replace `low'=$S_5 in `i'
                                        replace `high'=$S_6 in `i'
                                        local i=`i'+1
                                }
                        }
		}
		if "`logit'"!="" {
			label var `y' "logit Pr(`yvar'==`succode')"
			if "`ci'"!="" & "`exact'"=="" {
				gen `low'=`y'-`z'*`se'
				gen `high'=`y'+`z'*`se'
                        }
                        else if "`exact'"!="" {
                                replace `low'=ln(`low'/(1-`low'))
                                replace `high'=ln(`high'/(1-`high'))
                        }
		}
		else {
			label var `y' "Pr(`yvar'==`succode')"
			if "`ci'"!="" & "`exact'"=="" {
	                        tempvar logity
				gen `logity'=cond(`n'>0, ln((`r'+.5)/(`n'-`r'+.5)), .)
				gen `low'=`logity'-`z'*`se'
                                replace `low'=cond(`y'>0,exp(`low')/(1+exp(`low')),0)
				gen `high'=`logity'+`z'*`se'
                                replace `high'=cond(`y'<1,exp(`high')/(1+exp(`high')),1)
			}
		}
		if "`ci'"!="" & "`exact'"=="" {
			lab var `low' "Lower $S_level confidence limit"
			lab var `high' "Upper $S_level confidence limit"
		}
		else if "`exact'"!="" {
			lab var `low' "Exact lower $S_level confidence limit"
			lab var `high' "Exact upper $S_level confidence limit"
		}
	}
	graph `y' `low' `high' `collvar', /*
	 */ connect(`connect') symbol(`symbol') `xlall' `options'
end

*! version 1.2.0 22dec98 pr/arb
*! variation on xtile2 so that newvar is the median value within the quantile.
program define xtile3
        version 5.0
        local varlist "req new max(1)"
        local exp "req noprefix"
        local if "opt"
        local in "opt"
        local options "Nq(integer 2) ALL"
        tempvar new mnew
        parse "`*'"
        rename `varlist' `new'
        if `nq'<2 {
                di in red "nq() must be at least 2"
                exit 198
        }
        tempvar touse rhs
        qui gen double `rhs' = `exp'
        mark `touse' `if' `in'
        markout `touse' `rhs'
        quietly sum `rhs' if `touse'
	local nobs =_result(1)
        if `nq'>=`nobs' { error 2001 }      /* too few obs */
        quietly {
		replace `touse'=-`touse'
		sort `touse' `rhs'
		tempvar ind ctile ok
		gen long `ind'=int((int((_n-.5)*`nq'/`nobs')+1)*`nobs'/`nq')
		gen `ctile'=`rhs' if _n==`ind' & `touse'==-1
		gen byte `ok'= `ctile'!=.
		sort `rhs' `ok'
		replace `new'=sum(`ok'[_n-1])
                egen `mnew'=median(`rhs'), by(`new')
                if "`all'"=="" {
                        replace `mnew' = . if `touse'==0
                }
        }
        label var `mnew' "`nq' quantiles of `exp'"
        rename `mnew' `varlist'
end
