*! version 1.0.2   9mar00 arb
*! Reformats the output from regression commands
program define reformat
	version 6

	#delimit ;
	syntax [varlist (default=none)] [, noCOef noSE Z noP noCI noCONS n sd
	 iqr EForm DPCOef(int 3) dpp(int 3) noBRACkets to(str) DElimit(str)
	 sort LRtest NOISily];
	#delimit cr

	if `dpcoef' > 8 | `dpp' > 8 {
		disp in re "Maximum of 8 decimal places allowed"
		exit
	}
	if missing("`bracket'") {
		local opbrack (
		local clbrack )
	}
	if missing("`to'") {
		local to to
	}
	else if "`to'" == "no" {
		local to
	}
	local cmd = e(cmd)
	if "`cmd'" == "mlogit" {
		disp in red "not possible after mlogit"
		exit
	}
	else if "`cmd'" == "cox" & "`e(cmd2)'"!="stcox" & !missing("`lrtest'") {
		disp in red "lrtest option not available after cox - use stcox instead"
		exit
	}
	if !missing("`sd'") & !missing("`iqr'") {
		disp in red "Cannot specify both sd and iqr options"
		exit
	}
	qui `cmd'				/* refresh _results */
	if "`cmd'" == "logistic" { local eform eform }	/* turn on eform option automatically */
	if "`cmd'" == "regress" {
		local ztitle "t"
		local ptitle "P>|t|"
		local df = e(df_r)
		local zval = invt(`df',$S_level/100)
	}
	else {
		local ztitle "z"
		local ptitle "P>|z|"
		local df 100000000		/* makes t-distribution normal */
		local zval = -invnorm((100-$S_level)/200)
	}
	if missing("`eform'") {
		local estitle "     Coef."
		local estbase 0
	}
	else {
		local estbase 1
		local estfn exp
		if index("`cmd'","logi")>0 | "`e(fam)'" == "bin" {
			local estitle "Odds Ratio"
		}
		else if "`cmd'" == "poisson" | "`cmd'" == "nbreg" | "`e(fam)'" == "poi" | "`e(fam)'" == "nb" {
			local estitle "       IRR"
		}
		else if "`cmd'" == "cox" {
			local estitle "Haz. Ratio"
		}
		else {
			local estitle "exp(coef.)"
		}
	}
	tempname Cstar names
	matrix `Cstar' = get(VCE)		/* by default use all terms in the model */
	if "`cmd'" == "nbreg" {
		local nbc -1			/* omit the second constant for nbreg */
	}
	if ("`cons'" == "nocons" | "`eform'" == "eform") & "`cmd'" != "cox" {
		local coln = colsof(`Cstar')-1`nbc'		 /* ignore the constant */
	}
	else {
		local coln = colsof(`Cstar')`nbc'
	}
	matrix `names' = `Cstar'[1..`coln',1]
	local covl : rownames(`names')
	if missing("`varlist'") {
		local varlist `covl'
	}
	else {
		* Check user varlist is valid
		tokenize `varlist'
		while !missing("`1'") {
			local cl `covl'
			local fnd 0
			while !missing("`cl'") {
				gettoken cov cl: cl
				if "`1'"=="`cov'" {
					local fnd 1
					local cl
				}
			}
			if !`fnd' {
				disp in red "`1' not found in model"
				exit 111
			}
			mac shift
		}
	}
	if !missing("`sort'") {
		tokenize `varlist'
		local nvar : word count `varlist'
		local swap 1
		while `swap' {
			local i 1
			local ip1 2
			local swap 0
			while `i'<`nvar' {
				if lower("``i''") > lower("``ip1''") {
					* Swap element i and i+1
					local hold ``i''
					local `i' ``ip1''
					local `ip1' `hold'
					local swap 1
				}
				local i = `i'+1
				local ip1 = `ip1'+1
			}
		}
		local i 1
		local varlist
		while `i'<=`nvar' {
			local varlist "`varlist'``i'' "
			local i=`i'+1
		}
	}
	* Process labels and define categorical variables of all terms in model
	local vlist `covl'
	while !missing("`vlist'") {
		gettoken term vlist: vlist
		if "`term'"!="_cons" {
			* Stub, variable heading, and value label
			local varlab : variable label `term'
			if substr("`term'",1,1)=="I" {
				local i 1
				local varnme2
				local varlp2
				local var2
				local link
				local inter
				tokenize "`varlab'", parse("()==&*")
				while !missing("`1'") {
					cap confirm variable `1'
					if _rc == 0 {				/* a variable */
						local varnme`i' `1'
						local var`i' `1'		/* this will be over-written if categorical variable */
						local varlp`i' : variable label `varnme`i''
						if missing("`varlp`i''") {
							local varlp`i' `varnme`i''
						}
					}
					else {
						cap confirm integer number `1'
						if _rc == 0 {			/* a number */
							local varval`i' `1'
							local vallab`i': value label `varnme`i''
							if !missing("`vallab`i''") {
								local var`i': label `vallab`i'' `varval`i''
							}
							else {
								local var`i' `varval`i''
							}
							local i = `i' + 1
						}
						else if "`1'" == "&" | "`1'" == "*" {
							local link " `1' "
							local inter " interaction"
						}
					}
					mac shift
				}
				char `term'[vstub] `varnme1'`varnme2'					/* cat variable id */
				char `term'[var2] `varnme2'								/* interaction variable (needed for mult) */
				char `term'[varlab] `varlp1'`link'`varlp2'`inter'		/* variable description */
				char `term'[vallab] `var1'`link'`var2'					/* value label */
				* Baseline group for this term (except interactions)
				if missing("`inter'") {
					char `term'[cat] `varval1'							/* category */
					local basel : char `varnme1'[omit]
					local prev : char _dta[omit]
					if missing("`basel'") & missing("`prev'") {
						qui summ `varnme1'
						local basel = r(min)
					}
					else {
						/* most prevalent group code needed here */
					}
					* Value label
					if !missing("`vallab1'") {
						local var: label `vallab1' `basel'
					}
					else {
						local var `basel'
					}
					char `term'[bascat] `basel'							/* baseline category */
					char `term'[basvl] `var'							/* baseline value label */
				}
				* N for this term
				if !missing("`n'") {
					if index("`varlab'","*") > 0 {
						local varlab = substr("`varlab'",1,index("`varlab'","*")-1)
					}
					qui count if `varlab' & e(sample)
					char `term'[nobs] `r(N)'							/* N */
				}
			}
			else {
				char `term'[vstub] `term'
                if missing("`varlab'") {
                    local varlab `term'
                }
				char `term'[varlab] `varlab'
				if !missing("`sd'") {
					qui summ `term'
					local mult=r(sd)
					local stdev : display %6.3g `r(sd)'
					local var "per S.D. (=`stdev')"
				}
				else if !missing("`iqr'") {
					qui summ `term', detail
					local mult = r(p75)-r(p25)
					local iqr : display %6.3g `mult'
					local var "per IQR (=`iqr')"
				}
				else {
					local var "per unit"
					local mult 1
				}
				char `term'[vallab] `var'
				char `term'[mult] `mult'								/* Beta multiplier */
				char `term'[nobs] `e(N)'
			}
		}
	}

	if !missing("`lrtest'") {
		local ptitle "P^   "
		tempvar es
		gen byte `es'=e(sample)
		disp in blue "Performing likelihood-ratio tests" _c
		local dpv `e(depvar)'
		lrtest, saving(0)
		cap estimates unhold orig
		estimates hold orig
		local ovstub
		tokenize `varlist'
		while !missing("`1'") {
			if "`1'"!="_cons" {
				local vstub: char `1'[vstub]
				if "`vstub'"!="`ovstub'" {
					local ovstub `vstub'
					local cl `covl'
					local fitl
					while !missing("`cl'") {
						gettoken cov cl: cl
						if "`cov'"!="_cons" {
							local covst: char `cov'[vstub]
							if "`vstub'"!="`covst'" {
								local fitl "`fitl' `cov'"
							}
						}
					}
					disp in blue "." _c
					qui {
					if "`cmd'"=="cox" {
						stcox `fitl' if `es'
					}
					else {
						`cmd' `dpv' `fitl' if `es'
					}
					`noisily' disp in blue _n "`vstub'"
					`noisily' lrtest
					}	/* end quietly */
					char `1'[lrpval] `r(p)'								/* Likelihood ratio P-value */
				}
			}
			mac shift
		}
		estimates unhold orig
		disp
	}

	disp in gr _n upper("`cmd'") " `e(title_fl)' formatted output"
	local varlab: variable label `e(depvar)'
	if !missing("`varlab'") { local varlab "(`varlab')" }
	disp in gr _n "Outcome variable: " in ye "`e(depvar)'" in gr " `varlab', n=" in ye e(N)
	disp in gr _dup(78) "-"
	if missing("`delimit'") {
		global S_delim
		local opbrack " `opbrack'"
		local vartl "Covariate            "
		local ntl "    n      "
		local cotl "`estitle' "
		local setl " Std. Err. "
		local ztl "       `ztitle'   "
		local ptl "     `ptitle' "
		local citl "`opbrack'$S_level% Conf. Interval`clbrack'"
	}
	else {
		global S_delim "`delimit'"
		local vartl "Covariate$S_delim"
		local ntl "n$S_delim"
		local cotl =trim("`estitle'$S_delim")
		local setl "Std. Err.$S_delim"
		local ztl "`ztitle'$S_delim"
		local ptl =trim("`ptitle'")+"$S_delim"
		local citl "`opbrack'$S_level% Conf. Interval`clbrack'"
	}
	disp in gr "`vartl'" _c
	local lrp 22			/* spacing needed for LRtest P-value */
	local lrc 1				/* number of commas needed */
	if !missing("`n'") {
		disp in gr "`ntl'" _c
		local lrp=`lrp'+11
		local lrc=`lrc'+1
	}
	if missing("`coef'") {
		disp in gr "`cotl'" _c
		local lrp=`lrp'+11
		local lrc=`lrc'+1
	}
	if missing("`se'") {
		disp in gr "`setl'" _c
		local lrp=`lrp'+11
		local lrc=`lrc'+1
	}
	if !missing("`z'") {
		disp in gr "`ztl'" _c
		local lrp=`lrp'+11
		local lrc=`lrc'+1
	}
	if missing("`p'") | !missing("`lrtest'") {
		disp in gr "`ptl'" _c
	}
	if missing("`ci'") {
		disp in gr "`citl'" _c
	}
	disp
	if missing("`delimit'") {
		disp in gr _dup(78) "-"
	}

	************************************
	***			 MAIN LOOP			 ***
	************************************
	local ovstub					/* remember which vstub is being processed */
	tokenize "`varlist'"
	while !missing("`1'") {
		local mult 1
		cap confirm variable `1'	/* don't report cut points etc. */
		if _rc == 0 | "`1'" == "_cons" {
			* Variable description for `I' type categorical variables
			if substr("`1'",1,1)=="I" {
				local vstub: char `1'[vstub]
				cap local vstub2: char `2'[vstub]
				if "`vstub'"!="`ovstub'" {			/* first time round for this vstub */
					* Display variable description
					dispvd `1' `dpp' `lrp' `lrc' `lrtest'
					local ovstub `vstub'
					global S_base 0				/* Displayed baseline group flag */
				}
				* Need beta multiplier for interaction term?
				cap local var2: char `1'[var2]
				cap local mult: char `var2'[mult]
				if missing("`mult'") { local mult 1 }
				* Display baseline group now?
				* Yes if baseline is less than current group or at last group in current variable
				cap local basel : char `1'[bascat]
				if !missing("`basel'") & !$S_base {
					local varval : char `1'[cat]
					if `basel'<`varval' {
						local bef yes		/* print baseline immediately */
					}
					else if "`vstub'" != "`vstub2'" {
						local bef no		/* print baseline straight after this one */
					}
					if !missing("`bef'") {
						local desc: char `1'[basvl]
						if !missing("`n'") {
							qui count if `vstub'==`basel' & e(sample)
							local nval: display %5.0f `r(N)'
						}
						else {
							local nval -1
						}
						* Display baseline before current term?
						if "`bef'"=="yes" {
							dispbas "`desc'" `nval' `estbase' `coef'
							local bef
						}
						else {
							* Baseline description will be printed after this category
							local bef `""`desc'" `nval' `estbase' `coef'"'
						}
					}
				}
				* Display value label for this term
				local var: char `1'[vallab]
				if missing("$S_delim") {
					disp in gr "  `var'" _col(22) _c
				}
				else {
					disp in gr trim("`var'$S_delim") _c
				}
			}
			else {
				* Variable description for non I. (usually continuous) variables
				if "`1'" == "_cons" {
					local var "Constant"
					local nval `e(N)'
				}
				else {
					* Display variable description
					dispvd `1' `dpp' `lrp' `lrc' `lrtest'
					local var: char `1'[vallab]
					local var "  `var'"
					local mult: char `1'[mult]
				}
				if missing("$S_delim") {
					disp in gr "`var'" _col(22) _c
				}
				else {
					disp in gr trim("`var'$S_delim") _c
				}
			}
			* Coefficients, standard errors, Z-vals and CIs
			cap disp _b[`1']			/* check estimate exists */
			if _rc | _b[`1'] == 0 {
				disp in bl " (dropped)" _c
			}
			else {
				if !missing("`n'") {
                    if "`1'"=="_cons" {
                        local nval `e(N)'
                    }
                    else {
						local nval: char `1'[nobs]
                    }
					local ndi: display %5.0f `nval'
					ltdi "`ndi'"
				}
				if missing("`coef'") {
					local ndi: display %10.`dpcoef'f `estfn'(_b[`1']*`mult')
					ltdi "`ndi'"
				}
				if missing("`se'") {
					if _se[`1']==0 {
						disp in ye "         ." _col(12) _c
					}
					else {
						if !missing("`eform'") {
							local ndi : display %10.`dpcoef'f exp(_b[`1']*`mult')*_se[`1']*`mult'
							ltdi "`ndi'"
						}
						else {
							local ndi : display %10.`dpcoef'f _se[`1']*`mult'
							ltdi "`ndi'"
						}
					}
				}
				if !missing("`z'") {
					local ndi: display %10.`dpp'f _b[`1']/_se[`1']
					ltdi "`ndi'"
				}
				if !missing("`lrtest'") {
					if missing("`delimit'") {
						disp _col(12) _c
					}
					else {
						disp "$S_delim" _c
					}
				}
				else if missing("`p'") {
					local ndi: display %10.`dpp'f tprob(`df',abs(_b[`1']/_se[`1']))
					ltdi "`ndi'"
				}
				if missing("`ci'") {
					if _se[`1']==0 {
						disp in ye " ." _c
					}
					else {
						local cilo : display %10.`dpcoef'f `estfn'(_b[`1']*`mult'-`zval'*_se[`1']*`mult')
						local cihi : display %10.`dpcoef'f `estfn'(_b[`1']*`mult'+`zval'*_se[`1']*`mult')
						disp in ye "`opbrack'" trim("`cilo'") " `to' " trim("`cihi'") "`clbrack'" _c
					}
				}
			}
			disp
			if !missing(`"`bef'"') {
				* Display baseline group
				dispbas `bef'
				local bef
			}
		}
		mac shift
	}
	disp in gr _dup(78) "-"
	if !missing("`ovstub'") {
		disp in gr "* Baseline category"
	}
	if !missing("`lrtest'") {
		disp in gr "^ From likelihood ratio test"
	}
	* Tidy up
	global S_base
	global S_delim
end

program define ltdi
	version 6
	if `1' == 0 {
		local 1 = "<" + substr(trim("`1'"),1,length(trim("`1'"))-1) + "1"
		if missing("$S_delim") {
			disp substr("      ",1,10-length("`1'")) "`1'" _col(12) _c
		}
		else {
			disp "`1'$S_delim" _c
		}
	}
	else {
		if missing("$S_delim") {
			disp in ye "`1'" _col(12) _c
		}
		else {
			disp in ye trim("`1'")+"$S_delim" _c
		}
	}
end

program define dispvd		/* display variable description label */
	version 6
	args 1 dpp lrp lrc lrtest
	local varlab: char `1'[varlab]
	local vd = upper(substr("`varlab'",1,1))+substr("`varlab'",2,.)
	if missing("$S_delim") {
		disp in gr "`vd'" _col(`lrp') _c
	}
	else {
		disp in gr "`vd'" _c
		while `lrc'>0 & !missing("`lrtest'") {
			disp in gr "$S_delim" _c
			local lrc=`lrc'-1
		}
	}
	if !missing("`lrtest'") {
		local p : char `1'[lrpval]
		local ndi: display %10.`dpp'f `p'
		ltdi "`ndi'"
	}
	disp
end

* Printout baseline group line on screen
program define dispbas
	version 6
	args basel nval estbase coef
	if missing("$S_delim") {
		disp in gr "  `basel'*" _col(22) _c
		if `nval'>-1 {
			disp in ye "  `nval'" _col(12) _c
		}
		if missing("`coef'") {
			disp in ye %10.0f `estbase'
		}
		else {
			disp
		}
	}
	else {
		disp in gr "`basel'*$S_delim" _c
		if `nval'>-1 {
			disp in ye "`nval'$S_delim" _c
		}
		if missing("`coef'") {
			disp in ye "`estbase'"
		}
		else {
			disp
		}
	}
	global S_base 1
end
